; Armchair.ahk. Armchair's AutoHotkey script for window control.
; Revised by Dennis Damico.  See HtmlViewer.java.
; Based on Mouser v1.0, Nov-2006, by Adam Pash <adam.pash@gmail.com>
; lifehacker.com/software/mouser/hack-attack-operate-your-mouse-with-your-keyboard-212816.php
; lifehacker.com/assets/resources/2006/11/mouser6.ahk
; Creative Commons License, creativecommons.org/licenses/by-nc/3.0/

; Command line optional arguments: Armchair.ahk fontsize linethickness

; Reload the script if invoked when already running.
#SingleInstance,Force

; Set delay that will occur after each windowing command.
; 0 will yield timeslice to other proceses.
SetWinDelay,0

; Set coordinate mode for mouse commands to be relative to entire screen.
CoordMode, Mouse

; Remember mouse position of latest left click for use by Left Click Again.
myMouse_x := -1
myMouse_y := -1

; Get screen size to myScreen_*
SysGet, myScreen_, Monitor
myScreen_width  := myScreen_Right
myScreen_height := myScreen_Bottom

; Define the colors
color_red       := 0xFF0000
color_black     := 0x000000

; Define the menu items
menu_LClick     := "Left Click"
menu_Re_LClick  := "Left Click Again"
menu_LDClick    := "Left Double Click"
menu_RClick     := "Right Click"
menu_Undo       := "Undo"
menu_Cancel     := "Cancel"
menu_Exit       := "Exit"

; Define counters for hotkey alternative
firstTime := 0  ; Time of 1st button press
stopCount := 0  ; Number of button presses in time interval

; Set default font size and a drawing line thickness.
; Check for command line arguments to override defaults.
myFontSize := 24
myLineWidth := 2
Loop, %0%
{
    if (A_Index == 1) {
        myFontSize := %A_Index%
    }
    else if (A_Index == 2) {
        myLineWidth := %A_Index%
    }
}

; Define the hotkey. Send hotkeys to START.
; Do not use keyboard hook since script wants to see its own sent hotkeys.
hotkey=Escape
hotkeyStart()

; Wait for hotkeys.
return

;-----------------------------------------------
; Hot key alternative: 3 STOP button presses within 2 seconds.
; $=Require keyboard hook so this script will not see its own sent Media_Stops.
$Media_Stop:: 
    stopCount := stopCount + 1
    if (stopCount == 1) {       ; 1st button press. Remember time.
        firstTime := A_NOW
        Send {Media_Stop}
    }
    else if (stopCount == 2) {  ; 2nd button press. Check for timeout.
        timeDiff := A_NOW
        EnvSub, timeDiff, %firstTime%, seconds
        if (timeDiff > 2) {     ; timeout
            firstTime := A_NOW
            stopCount := 1
            Send {Media_Stop}
        }
    }
    else if (stopCount == 3) {  ; 3rd button press. Check for timeout.
        timeDiff := A_NOW
        EnvSub, timeDiff, %firstTime%, seconds
        if (timeDiff > 2) {     ; timeout
            firstTime := A_NOW
            stopCount := 1
            Send {Media_Stop}
        }
        else {                  ; not timeout, send hotkey.
            firstTime := 0
            stopCount := 0
            Send {%hotKey%}
        }
    }
    return

;-----------------------------------------------
; Hotkey pressed. Put up the focus rectangle.
START:
    hotkeyPass()        ; Send hotkeys to PASS while script runs.
    initializeFocus()   ; Put up the focus rectangle.
    positionPointer()
    return

;-----------------------------------------------
; Hotkey pressed while script is running. Remove focus rectangle and send
; hotkey to the window.
PASS:
    hotkeyOff()         ; Turn off hotkey.
    removeFocus()       ; Remove focus and menus.
    removeMenus()
    Send {%hotkey%}     ; Send the hotkey to window.
    hotkeyStart()       ; Send next hotkey to START.
    return

;-----------------------------------------------
; Send hotkey to the START label.
hotkeyStart() {
    global hotkey
    HotKey,%hotkey%,START,T1 On
}

;-----------------------------------------------
; Send hotkey to the PASS label.
hotkeyPass() {
    global hotkey
    HotKey,%hotkey%,PASS,T1 On
}

;-----------------------------------------------
; Disable the hotkey.
hotkeyOff() {
    global hotkey
    HotKey,%hotkey%,PASS,T1 Off
}

;-----------------------------------------------
; Handle navigation and selection for menu (gui,n) with menuMax items.
; Return true if user makes a choice, false if user hotkeys out.
processMenuInput(n, ByRef menu, menuMax) {
    global hotkey

    ; Menu items are 1..menuMax. Initially, no item is selected.
    menuChoice := 0
    Loop {
        ; Get user's input. L=max input length,
        ; {}...=list of input terminating keys, Matchlist = null
        Input, UserInput, L1, {Enter}{Up}{Down}{%hotkey%}

        ; UP key.
        IfInString, ErrorLevel, EndKey:Up
        {
            menuChoice := menuChoice - 1
            if (menuChoice < 1) {
                menuChoice := menuMax
            }
            GuiControl,%n%: Choose, menu, %menuChoice%
        }

        ; DOWN key.
        IfInString, ErrorLevel, EndKey:Down
        {
            menuChoice := menuChoice + 1
            if (menuChoice > menuMax) {
                menuChoice := 1
            }
            GuiControl,%n%: Choose, menu, %menuChoice%
        }

        ; ENTER key.
        IfInString, ErrorLevel, EndKey:Enter
        {
            ; Return true if user has made a menu choice, else ignore.
            if (menuChoice > 0) {
                return true
            }
        }

        ; HOT key.
        IfInString, ErrorLevel, EndKey:%hotkey%
        {
            removeFocus()       ; Remove focus and menus.
            removeMenus()
            Send {%hotkey%}     ; Send the hotkey to window.
            return false
        }
    }
}

;-----------------------------------------------
; Create and show popup menu (gui,7).
showPopupMenu() {
    global popupMenuSelected
    global menu_LClick
    global menu_LDClick
    global menu_RClick
    global menu_Re_LClick
    global menu_Undo
    global menu_Cancel
    global menu_Exit

    ; Create a new gui window for the menu
    createGuiWindow(7)

    ; Add a list box with menu choices.
    ; Keep popupMenuMax accurate as number of choices.
    popupMenuMax := 7
    Gui,7: Add, ListBox, vpopupMenuSelected r%popupMenuMax%
    GuiControl,7: , popupMenuSelected, %menu_LClick%
    GuiControl,7: , popupMenuSelected, %menu_Re_LClick%
    GuiControl,7: , popupMenuSelected, %menu_LDClick%
    GuiControl,7: , popupMenuSelected, %menu_RClick%
    GuiControl,7: , popupMenuSelected, %menu_Undo%
    GuiControl,7: , popupMenuSelected, %menu_Cancel%
    GuiControl,7: , popupMenuSelected, %menu_Exit%

    ; Show menu, act on choice.
    showGuiWindow(7)
    if (processMenuInput(7, popupMenuSelected, popupMenuMax)) {
        popupMenuAction()
    }
}

;-----------------------------------------------
; Handle popup menu choice (gui,7).
popupMenuAction() {
    global popupMenuSelected
    global menu_LClick
    global menu_LDClick
    global menu_RClick
    global menu_Re_LClick
    global menu_Undo
    global menu_Cancel
    global menu_Exit

    Gui,7: Submit
    Gui,7: Destroy
    if (popupMenuSelected == menu_LClick) {
        positionMouse()
        leftClick()
        quit()
    }
    else if (popupMenuSelected == menu_Re_LClick) {
        leftClickAgain()
        quit()
    }
    else if (popupMenuSelected == menu_LDClick) {
        positionMouse()
        leftDoubleClick()
        quit()
    }
    else if (popupMenuSelected == menu_RClick) {
        positionMouse()
        rightClick()
        quit()
    }
    else if (popupMenuSelected == menu_Undo) {
        popFocus()
        positionPointer()
    }
    else if (popupMenuSelected == menu_Cancel) {
        quit()
    }
    else if (popupMenuSelected == menu_Exit) {
        quitApp()
    }
    else {
        quit()  ; no item selected
    }
    return
}

;-----------------------------------------------
; Create a window with default options for gui,n.
createGuiWindow(n) {
    global myFontSize

    Gui,%n%: Destroy
    Gui,%n%: Font, cBlue s%myFontSize%, Verdana
    Gui,%n%: +AlwaysOnTop -Caption +LastFound +ToolWindow
    Gui,%n%: Color, EEAA99      ; arbitrary custom color to...
    WinSet, TransColor, EEAA99  ; ...make window transparent
}

;-----------------------------------------------
; Show window at standard location for gui,n
showGuiWindow(n) {
    global myScreen_width
    global myScreen_height

    menuX := myScreen_width/3   ; See MenuManager.java.
    menuY := myScreen_height/6
    Gui,%n%: Show, AutoSize x%menuX% y%menuY%
}

;-----------------------------------------------
; Initialize the focus rectangle and focus history objects.
initializeFocus() {
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height
    global myScreen_width
    global myScreen_height

    ; Set the current focus rectangle to the entire screen.
    myFocus_x := 0
    myFocus_y := 0
    myFocus_width := myScreen_width
    myFocus_height := myScreen_height

    ; Keep history of focus rectangles.
    initializeFocusHistory()
}

;-----------------------------------------------
; Set the focus rectangle history to empty.
; The focus rectangle history is a stack with most recent items first.
; Focus variables are delimited by comma.
; Sets of focus variables are delimited by semicolon.
initializeFocusHistory() {
    global myFocusHistory
    myFocusHistory := ""
}

;-----------------------------------------------
; Add the current focus rectangle to the history.
pushFocus() {
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height
    global myFocusHistory

    myFocusHistory := myFocus_x . "," . myFocus_y . ","
                    . myFocus_width . "," . myFocus_height . "," . ";"
                    . myFocusHistory
}

;-----------------------------------------------
; Set the focus rectangle from the most recent values in the focus history.
; Remove those most recent values from the history.
popFocus() {
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height
    global myFocusHistory

    ; Find the delimiter after the newest focus set.
    p := InStr(myFocusHistory, ";")
    if (p == 0) {
        return ; focus stack is empty.
    }

    ; Remove the most recent focus set from the history.
    thisSet := SubStr(myFocusHistory, 1, p-1)
    myFocusHistory := SubStr(myFocusHistory, p+1)

    ; Set the focus from thisSet
    Loop, parse, thisSet, `, , ";"
    {
        if (A_Index == 1) {
            myFocus_x := A_LoopField
        }
        else if (A_Index == 2) {
            myFocus_y := A_LoopField
        }
        else if (A_Index == 3) {
            myFocus_width := A_LoopField
        }
        else if (A_Index == 4) {
            myFocus_height := A_LoopField
        }
    }
}

;-----------------------------------------------
; Position the cross hairs cursor and focus rectangle as directed by user.
positionPointer() {
    global hotkey
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height
    global myScreen_width
    global myScreen_height
    global myLineWidth

    ; Position mouse to focus center
    positionMouse()

    Loop {
        ; Draw the focus rectangle and cross hairs.
        drawFocus()

        ; Get user's input. L=max input length,
        ; {}...=list of input terminating keys, Matchlist = null
        Input, UserInput, L1, {Enter}{Left}{Right}{Up}{Down}{%hotkey%}

        ; UP key.
        IfInString, ErrorLevel, EndKey:Up
        {
            pushFocus()
            myFocus_height := Math_max(1, myFocus_height / 2)
        }

        ; DOWN key.
        IfInString, ErrorLevel, EndKey:Down
        {
            pushFocus()
            myFocus_height := Math_max(1, myFocus_height / 2)
            myFocus_y := myFocus_y + myFocus_height
        }

        ; LEFT key.
        IfInString, ErrorLevel, EndKey:Left
        {
            pushFocus()
            myFocus_width := Math_max(1, myFocus_width / 2)
        }

        ; RIGHT key.
        IfInString, ErrorLevel, EndKey:Right
        {
            pushFocus()
            myFocus_width := Math_max(1, myFocus_width / 2)
            myFocus_x := myFocus_x + myFocus_width
        }

        ; ENTER key.
        IfInString, ErrorLevel, EndKey:Enter
        {
            showPopupMenu()
            return
        }

        ; HOT key.
        IfInString, ErrorLevel, EndKey:%hotkey%
        {
            removeFocus()       ; Remove focus and menus.
            removeMenus()
            Send {%hotkey%}     ; Send the hotkey to window.
            return
        }
    }
    return
}

;-----------------------------------------------
; Draw the focus rectangle and crosshairs (gui,1-6).
drawFocus() {
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height
    global myLineWidth
    global color_red
    global color_black

    ; Crosshairs alternative 1: size of window.
    ; Draw horizontal and vertical crosshairs, window lengths.
    drawLine(1, "H", color_red, 0, myFocus_y + myFocus_height/2 - myLineWidth/2)
    drawLine(2, "V", color_red, myFocus_x + myFocus_width/2 - myLineWidth/2, 0)

    ; Draw the top, bottom, left, and right sides of the focus rectangle
    drawLine(3, "h", color_black, myFocus_x, myFocus_y)
    drawLine(4, "h", color_black, myFocus_x, myFocus_y + myFocus_height - myLineWidth)
    drawLine(5, "v", color_black, myFocus_x, myFocus_y)
    drawLine(6, "v", color_black, myFocus_x + myFocus_width - myLineWidth, myFocus_y)

    positionMouse()
}

;-----------------------------------------------
; Draw a horizontal or vertical line on the screen over all other objects.
drawLine(n, theStyle, theColor, theX, theY) {
    global myFocus_width
    global myFocus_height
    global myLineWidth
    global myScreen_width
    global myScreen_height

    if (theStyle=="H") {
        ; horizontal line, window width
        thisWidth := myScreen_width
        thisHeight := myLineWidth
    }
    else if (theStyle=="h") {
        ; horizontal line, focus width
        thisWidth := myFocus_width
        thisHeight := myLineWidth
    }
    else if (theStyle=="V") {
        ; vertical line, window height
        thisWidth := myLineWidth
        thisHeight := myScreen_height
    }
    else if (theStyle=="v") {
        ; vertical line, focus height
        thisWidth := myLineWidth
        thisHeight := myFocus_height
    }
    Gui,%n%: +AlwaysOnTop -Caption +LastFound +ToolWindow
    Gui,%n%: Color, %theColor%
    Gui,%n%: Show, x%theX% y%theY% w%thisWidth% h%thisHeight%
}

;-----------------------------------------------
; Position the mouse to the center of the focus rectangle.
positionMouse() {
    global myFocus_x
    global myFocus_y
    global myFocus_width
    global myFocus_height

    MouseMove myFocus_x + myFocus_width/2, myFocus_y + myFocus_height/2
}

;-----------------------------------------------
; Return larger of parameters.
Math_max(a,b) {
    if (a>b) return a
    return b
}

;-----------------------------------------------
; Remove focus, initialize hotkey, and exit.
quit() {
    removeFocus()
    removeMenus()
    hotkeyStart()
    Exit
}

;-----------------------------------------------
; Simulate a left mouse click at the crosshairs position.
; Remember that position in myMouse_*
leftClick() {
    global myMouse_x
    global myMouse_y

    removeFocus()
    MouseGetPos, myMouse_x, myMouse_y
    Click
}

;-----------------------------------------------
; Repeat a left mouse click at the previous click position.
leftClickAgain() {
    global myMouse_x
    global myMouse_y

    if (myMouse_x >= 0) {
        MouseMove, myMouse_x, myMouse_y
        Click
    }
}

;-----------------------------------------------
; Simulate a left mouse double click at the crosshairs position.
leftDoubleClick() {
    removeFocus()
    Click 2
}

;-----------------------------------------------
; Simulate a right mouse click at the crosshairs position.
rightClick() {
    removeFocus()
    Click right
}

;-----------------------------------------------
; Remove the focus rectangle and crosshairs.
removeFocus() {
    Gui,1: Destroy
    Gui,2: Destroy
    Gui,3: Destroy
    Gui,4: Destroy
    Gui,5: Destroy
    Gui,6: Destroy
}

;-----------------------------------------------
; Remove the gui menus.
removeMenus() {
    Gui,7: Destroy
}

;-----------------------------------------------
; Exit the AutoHotkey application.
quitApp() {
    ExitApp
}
